% computes preconditioned eigenvalue distribution for scaled mass matrices
% when AMG is used instead of the full vector Laplace matrix, cf.
% EigTables.m
% based on: XINFSUP
%   IFISS scriptfile: DJS; 27 May 2015.
% Copyright (c) 2005 D.J. Silvester, H.C. Elman, A. Ramage
%
% Edited JJP: 22 Jan, 2016.

% Adds necessary directories to Matlab path
thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

global amg_grid amg_smoother number_vcycles

problist = {'S3_G4_R3';'S5_G4'};
numprob = length(problist);
elemlist = {'Q1Q1','Q1P0','Q2Q1'};
numelem = length(elemlist);
diag_type = 'diag'; % diag or lump

% Choice of scaling parameter alpha
alphlist = [1:10 20:20:100];
numscale = length(alphlist);


for k = 1:numprob
    pb = problist{k};
    display(pb)
    eigmatdiag = zeros(numelem,numscale,4); % Eigenvalues with H=diag(Q)/lump(Q)
    eigmatfull = zeros(numelem,numscale,4); % Eigenvalues with H=Q
    boundfull = zeros(numelem,numscale);    % Bound (3.1) with H=diag(Q)/lump(Q)
    bounddiag = zeros(numelem,numscale);    % Bound (3.1) with H = Q
    
    for j = 1:numelem
        pbel = [pb,'_',elemlist{j}];
        batchmode_scalestokes(pbel); % Need this for obstacle prob
        close all;
        load batchrun
        cd(thisdir)
        
        [np,nu]=size(Bst); ndof=nu/2;
        dof=(1:ndof)';
        
        A=Ast;
        A = 0.5*(A + A');
        B=Bst;
        
        if qmethod < 2
            K = [A,B';B,-beta*C];
        else
            C = sparse(np,np);
            K = [A,B';B,sparse(np,np)];
        end
        K = 0.5*(K+K');
        
        % Mass matrix and diagonal/lumped approximation
        QQ=0.5*(Q+Q');
        switch diag_type
            case 'diag'
                QQd = diag(diag(Q));
            case 'lump'
                QQd = diag(sum(Q,2));
            otherwise
                QQd = diag(diag(Q));
        end
        
        % AMG Inverse
        fprintf('Computing AMG inverse matrix...')
        Ainv = sparse(nu,nu); I = speye(nu);
        number_vcycles = 1;
        amg_grid = amg_grids_setup(A);
        fprintf('AMG with point damped Gauss-Seidel smoothing and 1 V-cycle ..\n')
        smoother_params = amg_smoother_params(amg_grid, 'PGS', 2);
        amg_smoother = amg_smoother_setup(amg_grid, smoother_params);
        aparams = struct('A',A);
        
        for A_ind = 1:nu
            Ainv(:,A_ind) = m_amgzz(I(:,A_ind),aparams);
        end
        fprintf('Done\n');
        
        % Compute eigenvalues of P_alpha\A
        fprintf('Computing eigenvalues...')
        for i = 1:numscale;
            fprintf('alpha = %3.2g\n',alphlist(i));
            P_alph = [Ainv,0*B';0*B,inv(alphlist(i)*QQd)];
            P_fullalph = [Ainv,0*B';0*B,inv(alphlist(i)*Q)];
            e = eig(full(P_alph*K));
            ef = eig(full(P_fullalph*K));
            e(abs(e) < 1e-14) = []; ef(abs(ef) < 1e-14) = [];
            e=sort(real(e));
            eneg= e(e<0); epos= e(e>0);
            efneg = ef(ef<0); efpos = ef(ef>0);
            eigmatdiag(j,i,:) = [min(eneg), max(eneg), min(epos), max(epos)];
            eigmatfull(j,i,:) = [min(efneg), max(efneg), min(efpos), max(efpos)];
        end
    end
    
    % Write tables
    cd(thisdir)
    fprintf('Writing tables ...')
    filename_diag = ['AMG',pb,'EigDiagBound.tex'];
    filename_full = ['AMG',pb,'EigFullBound.tex'];
    
    fid_diag = fopen(filename_diag,'w+');
    fid_full = fopen(filename_full,'w+');
    
    fprintf(fid_diag,'& \\multicolumn{4}{|c|}{\\qqel{}} & \\multicolumn{4}{|c|}{\\qpel{}} & \\multicolumn{4}{|c}{\\qqqel{}}\\\\\n');
    fprintf(fid_full,'& \\multicolumn{4}{|c|}{\\qqel{}} & \\multicolumn{4}{|c|}{\\qpel{}} & \\multicolumn{4}{|c}{\\qqqel{}}\\\\\n');
    
    fprintf(fid_diag,'$\\alpha$ & $\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+1}$ & $\\lambda_{m+n}$ & ');
    fprintf(fid_diag,'$\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+1}$ & $\\lambda_{m+n}$ & ');
    fprintf(fid_diag,' $\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+1}$ & $\\lambda_{m+n}$ \\\\\n');
    
    fprintf(fid_full,'$\\alpha$ & $\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+1}$ & $\\lambda_{m+n}$ & ');
    fprintf(fid_full,'$\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+1}$ & $\\lambda_{m+n}$ & ');
    fprintf(fid_full,' $\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+1}$ & $\\lambda_{m+n}$ \\\\\n');
    
    fprintf(fid_diag,'\\hline\n');
    fprintf(fid_full,'\\hline\n');
    
    
    for i = 1:numscale
        fprintf(fid_diag,'%i & %3.2g & %3.2g & %3.2g & %3.2g & ',alphlist(i),eigmatdiag(1,i,:));
        fprintf(fid_diag,'%3.2g & %3.2g & %3.2g & %3.2g & ',eigmatdiag(2,i,:));
        fprintf(fid_diag,'%3.2g & %3.2g & %3.2g & %3.2g \\\\\n',eigmatdiag(3,i,:));
        fprintf(fid_full,'%i & %3.2g & %3.2g & %3.2g & %3.2g & ',alphlist(i),eigmatfull(1,i,:));
        fprintf(fid_full,'%3.2g & %3.2g & %3.2g & %3.2g & ',eigmatfull(2,i,:));
        fprintf(fid_full,'%3.2g & %3.2g & %3.2g & %3.2g \\\\\n',eigmatfull(3,i,:));
    end
    fprintf('Done\n');
    
    fprintf('Saving results...')
    save(['AMG',pb,'Eigs.mat'],'alphlist','eigmatdiag','eigmatfull');
    fprintf('Done\n');
end
fclose all;