% computes preconditioned eigenvalue distribution for scaled mass matrices
% Computes Tables 3.1, 3.2, 3.4 and 3.5 in Pearson, Pestana & Silvester,
% 2016.
% based on: XINFSUP
%   IFISS scriptfile: DJS; 27 May 2015.
% Copyright (c) 2005 D.J. Silvester, H.C. Elman, A. Ramage
%
% Edited JJP: 22 Jan, 2016. Loop over matrices; get extreme eigs; tables
% Edited JJP: 20 Apr, 2017. Changed to different bounds. 

% Adds necessary directories to Matlab path
thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

problist = {'S3_G5_R3'};
numprob = length(problist);
elemlist = {'Q1Q1','Q1P0','Q2Q1'};
numelem = length(elemlist);
includebound = 1; % Include boundary nodes
diag_type = 'diag'; % diag or lump

% Choice of scaling parameter alpha
alphlist = [1:10 20:20:100];
numscale = length(alphlist);

for k = 1:numprob
    pb = problist{k};
    eigmatdiag = zeros(numelem,numscale,4); % Eigenvalues with H=diag(Q)/lump(Q)
    eigmatfull = zeros(numelem,numscale,4);  % Eigenvalues with H=Q
    boundfull = zeros(numelem,numscale);    % Bound (3.1) with H=diag(Q)/lump(Q)
    bounddiag = zeros(numelem,numscale);    % Bound (3.1) with H = Q
    gam = zeros(numelem,2);
    del = zeros(numelem,2);
    the = zeros(numelem,1);
    for j = 1:3
        pbel = [pb,'_',elemlist{j}];
        batchmode_scalestokes(pbel); % Need this for obstacle prob
        close all;
        load batchrun
        cd(thisdir)
        
        [np,nu]=size(Bst); ndof=nu/2;
        dof=(1:ndof)';
        
        if includebound == 0 % Remove boundary nodes if required
            inner=setdiff(dof,bound);
            A=Ast([inner,inner+ndof],[inner,inner+ndof]);
            B=Bst(:,[inner,inner+ndof]);
        else
            A=Ast;
            B=Bst;
        end
        
        A = 0.5*(A + A');
        
        if qmethod < 2
            K = [A,B';B,-beta*C];
        else
            C = sparse(np,np);
            K = [A,B';B,sparse(np,np)];
        end
        K = 0.5*(K+K');
        
        QQ=0.5*(Q+Q');
        switch diag_type
            case 'diag'
                QQd = diag(diag(Q));
            case 'lump'
                QQd = diag(sum(Q,2));
            otherwise
                QQd = diag(diag(Q));
        end
        
        % Compute information for bound (3.1)
        r = rank(full(B'));
        [~,~,V] = svd(full(B'));
        Z = V(:,1:r);
        del_full = eig(Z'*full(B*(A\(B'))+beta*C)*Z,Z'*full(QQ)*Z);
        del_full = sort(del_full,'ascend');
        del_full = del_full(1);
        del_diag = eig(Z'*full(B*(A\(B'))+beta*C)*Z,Z'*full(QQd)*Z);
        del_diag = sort(del_diag,'ascend');
        del_diag = del_diag(1);
        the_diag = eig(full(QQ),full(QQd));
        the_diag = min(the_diag);
        gam_full = eig(Z'*full(beta*C)*Z,Z'*full(QQ)*Z);
        gam_diag = eig(Z'*full(beta*C)*Z,Z'*full(QQd)*Z);
        if qmethod < 2
            gam_full = sort(gam_full,'descend');
            gam_full = gam_full(1);
            gam_diag = sort(gam_diag,'descend');
            gam_diag = gam_diag(1);
        else
            gam_full = 0;
            gam_diag = 0;
        end
        gam(j,:) = [gam_full, gam_diag];
        the(j) = the_diag;
        del(j,:) = [del_full, del_diag];
        
        % Compute eigenvalues of P_alpha\A
        fprintf('Computing eigenvalues ...\n')
        for i = 1:numscale;
            fprintf('alpha = %3.2g\n',alphlist(i));
            P_alph = [A,0*B';0*B,alphlist(i)*QQd];
            P_fullalph = [A,0*B';0*B,alphlist(i)*Q];
            e = eig(full(K),full(P_alph));
            ef = eig(full(K),full(P_fullalph));
            e(abs(e) < 1e-14) = []; ef(abs(ef) < 1e-14) = [];
            [e]=sort(real(e));
            eneg= e(e<0); epos= e(e>0);
            efneg = ef(ef<0); efpos = ef(ef>0);
            eigmatdiag(j,i,:) = [min(eneg), max(eneg), min(epos), max(epos)];
            eigmatfull(j,i,:) = [min(efneg), max(efneg), min(efpos), max(efpos)];
            boundfull(j,i) = (1-sqrt(1 + 4*del_full/alphlist(i)))/2;
            bounddiag(j,i) = (1-sqrt(1 + 4*del_full/alphlist(i)*the_diag))/2;
        end
    end % for j = 1:numelem
    
    
    % Write tables
    fprintf('Writing tables ...')
    if includebound == 0
        filename_diag = [pb,'EigDiagInt.tex'];
        filename_full1 = [pb,'EigFullInt1.tex'];
        filename_full3 = [pb,'EigFullInt2.tex'];
        filename_diag_bound = [pb,'EigBoundDiagInt.tex'];
        filename_full_bound = [pb,'EigBoundFullInt.tex'];
    else
        filename_diag = [pb,'EigDiag.tex'];
        filename_full = [pb,'EigFull.tex'];
        filename_diag_bound = [pb,'EigBoundDiag.tex'];
        filename_full_bound = [pb,'EigBoundFull.tex'];
    end
    
    fid_diag = fopen(filename_diag,'w+');
    fid_full = fopen(filename_full,'w+');
    fid_diag_bound = fopen(filename_diag_bound,'w+');
    fid_full_bound = fopen(filename_full_bound,'w+');
    
    fprintf(fid_diag,'& \\multicolumn{3}{|c|}{\\qqel{}} & \\multicolumn{3}{|c}{\\qqqel{}}\\\\\n');
    fprintf(fid_full1,'& \\multicolumn{3}{|c|}{\\qqel{}} & \\multicolumn{3}{|c}{\\qpel{}} \\\\\n');
    fprintf(fid_full2,' & \\multicolumn{3}{|c}{\\qqqel{}}\\\\\n');
    
    fprintf(fid_diag,'$\\alpha$ & $\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+n}$ & ');
    fprintf(fid_diag,' $\\lambda_{1}$ & $\\lambdam$ & $\\lambda_{m+n}$ \\\\\n');
    
    fprintf(fid_full,'$\\alpha$ & $\\lambda_{1}$ & $\\lambdam$ &  $\\lambda_{m+n}$ & ');
    fprintf(fid_full,'$\\lambda_{1}$ & $\\lambdam$ &  $\\lambda_{m+n}$ & ');
    fprintf(fid_full,'  $\\lambda_{1}$ & $\\lambdam$ &  $\\lambda_{m+n}$ \\\\\n');
    
    fprintf(fid_diag,'\\hline\n');
    fprintf(fid_full,'\\hline\n');
    
    fprintf(fid_diag_bound,'& \\multicolumn{2}{|c|}{\\qqel{}} & \\multicolumn{2}{|c}{\\qqqel{}}\\\\\n');
    fprintf(fid_full_bound,'& \\multicolumn{2}{|c|}{\\qqel{}} & \\multicolumn{2}{|c|}{\\qpel{}} & \\multicolumn{2}{|c}{\\qqqel{}}\\\\\n');
    
    fprintf(fid_diag_bound,'$\\alpha$ & $\\lambdam$ & Bound& $\\lambdam$ & Bound\\\\\n');
    fprintf(fid_full_bound,'$\\alpha$ & $\\lambdam$ & Bound & $\\lambdam$ & Bound & $\\lambdam$ & Bound\\\\\n');
    
    fprintf(fid_diag_bound,'\\hline\n');
    fprintf(fid_full_bound,'\\hline\n');
    
    %%% Get floating point representations %%%
    [eigmatdiag_exp,eigmatdiag_man] = floating_point(eigmatdiag);
    [eigmatfull_exp,eigmatfull_man] = floating_point(eigmatfull);
    [bounddiag_exp,bounddiag_man] = floating_point(bounddiag);
    [boundfull_exp,boundfull_man] = floating_point(boundfull);
    
    
    for i = 1:numscale
        fprintf(fid_diag,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & ',alphlist(i),eigmatdiag_man(1,i,1),eigmatdiag_exp(1,i,1),eigmatdiag_man(1,i,2),eigmatdiag_exp(1,i,2),eigmatdiag(1,i,4));
        fprintf(fid_diag,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g \\\\\n ',eigmatdiag_man(3,i,1),eigmatdiag_exp(3,i,1),eigmatdiag_man(3,i,2),eigmatdiag_exp(3,i,2),eigmatdiag(3,i,4));
        fprintf(fid_full,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & ',alphlist(i),eigmatfull_man(1,i,1),eigmatfull_exp(1,i,1),eigmatfull_man(1,i,2),eigmatfull_exp(1,i,2),eigmatfull(1,i,4));
        fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & ',eigmatfull_man(2,i,1),eigmatfull_exp(2,i,1),eigmatfull_man(2,i,2),eigmatfull_exp(2,i,2),eigmatfull(2,i,4));
        fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g \\\\\n ', eigmatfull_man(3,i,1),eigmatfull_exp(3,i,1),eigmatfull_man(3,i,2),eigmatfull_exp(3,i,2),eigmatfull(3,i,4));
        fprintf(fid_diag_bound,'%i &  \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} &  \\fp{%3.2g}{%i} &  \\fp{%3.2g}{%i}\\\\\n',alphlist(i),eigmatdiag_man(1,i,2),eigmatdiag_exp(1,i,2),bounddiag_man(1,i),bounddiag_exp(1,i),eigmatdiag_man(3,i,2),eigmatdiag_exp(3,i,2),bounddiag_man(3,i),bounddiag_exp(3,i));
        fprintf(fid_full_bound,'%i &  \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} &  \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} &  \\fp{%3.2g}{%i} &  \\fp{%3.2g}{%i}\\\\\n',alphlist(i),eigmatfull_man(1,i,2),eigmatfull_exp(1,i,2),boundfull_man(1,i),boundfull_exp(1,i),eigmatfull_man(2,i,2),eigmatfull_exp(2,i,2),boundfull_man(2,i),boundfull_exp(2,i),eigmatfull_man(3,i,2),eigmatfull_exp(3,i,2),boundfull_man(3,i),boundfull_exp(3,i));
    end
    fprintf('Done\n');
    
    fprintf('Saving results...')
    save([pb,'Eigs.mat'],'alphlist','eigmatdiag','eigmatfull','bounddiag','boundfull','gam','the','del','diag_type');
    fprintf('Done\n');
end % for k = 1:numprob

fclose all;