% computes preconditioned eigenvalue distribution for scaled mass matrices
% Computes Tables 3.1, 3.2, 3.4 and 3.5 in Pearson, Pestana & Silvester,
% 2016.
% based on: XINFSUP
%   IFISS scriptfile: DJS; 27 May 2015.
% Copyright (c) 2005 D.J. Silvester, H.C. Elman, A. Ramage
%
% Edited JJP: 22 Jan, 2016. Loop over matrices; get extreme eigs; tables
% Edited JJP: 20 Apr, 2017. Changed bound

% Adds necessary directories to Matlab path
thisdir = pwd;
addpath(genpath(thisdir));
setifisspath;
cd(ifisspath)
setpath;
cd(thisdir)

problist = {'S3_G5_R3'};
Gammalist = [1];
numprob = length(problist);
elemlist = {'Q1Q1','Q1P0','Q2Q1'};
includebound = 1;
numelem = length(elemlist);
diag_type = 'diag'; % diag or lump

% Choice of scaling parameter alpha
alphlist = [1:10 20:20:100];
numscale = length(alphlist);

for k = 1:numprob
    pb = problist{k};
    eigmatdiag = zeros(numelem,numscale,4); % Eigenvalues with H=diag(Q)/lump(Q)
    eigmatfull = zeros(numelem,numscale,4);  % Eigenvalues with H=Q
    dellist = zeros(numelem,2);
    minresfaclist = zeros(numelem,numscale,2);
    for j = 1:numelem
        pbel = [pb,'_',elemlist{j}];
        batchmode_scalestokes(pbel); % Need this for obstacle prob
        close all;
        load batchrun
        cd(thisdir)
        
        [np,nu]=size(Bst); ndof=nu/2;
        dof=(1:ndof)';
        
        if includebound == 0 % Remove boundary nodes if required
            inner=setdiff(dof,bound);
            A=Ast([inner,inner+ndof],[inner,inner+ndof]);
            B=Bst(:,[inner,inner+ndof]);
        else
            A=Ast;
            B=Bst;
        end
        
        A = 0.5*(A + A');
        
        if qmethod < 2
            K = [A,B';B,-beta*C];
        else
            C = sparse(np,np);
            K = [A,B';B,sparse(np,np)];
        end
        K = 0.5*(K+K');
        
        QQ=0.5*(Q+Q');
        switch diag_type
            case 'diag'
                QQd = diag(diag(Q));
            case 'lump'
                QQd = diag(sum(Q,2));
            otherwise
                QQd = diag(diag(Q));
        end
        
        % Compute information for bound (3.1)
        r = rank(full(B'));
        [~,~,V] = svd(full(B'));
        Z = V(:,1:r);
        del_full = eig(Z'*full(B*(A\(B'))+beta*C)*Z,Z'*full(QQ)*Z);
        del_full = sort(del_full,'ascend');
        del_full = del_full(1);
        del_diag = eig(Z'*full(B*(A\(B'))+beta*C)*Z,Z'*full(QQd)*Z);
        del_diag = sort(del_diag,'ascend');
        del_diag = del_diag(1);
        the_diag = eig(full(QQ),full(QQd));
        the_diag = min(the_diag);
        dellist(k,:) = [del_full del_diag];
        display(qmethod);
        
        
        % Compute eigenvalues of P_alpha\A
        fprintf('Computing eigenvalue bounds ...\n')
        for i = 1:numscale;
            alph = alphlist(i);
            
            fprintf('alpha = %3.2g\n',alph);
            if qmethod > 1
                Gam = Gammalist(k);
                af = (sqrt(1 + 4*Gam/alph)-1)/2;
                bf = (sqrt(1 + 4*del_full/alph)-1)/2; 
                cf = 1;
                df = (1 + sqrt(1+4*Gam/alph))/2;
                
                ad = (sqrt(1 + 9*Gam/alph)-1)/2;
                bd = (sqrt(1 + del_full/alph)-1)/2;
                ccd = 1;
                dd = (1 + sqrt(1+9*Gam/alph))/2;
            else
                Gam = Gammalist(k) + 1;
                afac = 1-1/alph;
                af = (sqrt(afac^2 + 4*Gam/alph)-afac)/2;
                bf = (sqrt(1 + 4*del_full/alph)-1)/2;
                cf = 1;
                df = (1 + sqrt(1+4*Gam/alph))/2;
                
                if qmethod == 0
                    afac = 1-9/(4*alph);
                    ad = (sqrt(afac^2 + 9*Gam/alph)-afac)/2;
                    bd = 0.25/alph;
                    ccd = 1;
                    dd = (1 + sqrt(1+9*Gam/alph))/2;
                end
                
                if qmethod == 1
                    ad = af; bd = bf; ccd = cf; dd = df;
                end
            end
            
            
            eigmatfull(j,i,:) = [af bf cf df];
            eigmatdiag(j,i,:) = [ad bd ccd dd];
            if af-bf > df-cf
               df = cf + (af-bf); 
            else
                af = bf + (df-cf);
            end
            minresfaclist(j,i,1) = (sqrt(af*df) - sqrt(bf*cf))/(sqrt(af*df) + sqrt(bf*cf));
            
            if ad-bd > dd-ccd
               dd = ccd + (ad-bd); 
            else
                ad = bd + (dd-ccd);
            end
            minresfaclist(j,i,2) = (sqrt(ad*dd) - sqrt(bd*ccd))/(sqrt(ad*dd) + sqrt(bd*ccd));
            
        end
    end % for j = 1:numelem
    cd(thisdir)
    
    % Write tables
    fprintf('Writing tables ...')
    if includebound == 0
        filename_diag = [pb,'MINRESEigDiagInt.tex'];
        filename_full = [pb,'MINRESEigFullInt.tex'];
        
    else
        filename_diag = [pb,'MINRESEigDiag.tex'];
        filename_full = [pb,'MINRESEigFull.tex'];
        
    end
    
    fid_diag = fopen(filename_diag,'w+');
    fid_full = fopen(filename_full,'w+');

    fprintf(fid_diag,'& \\multicolumn{3}{|c|}{\\qqel{}} & \\multicolumn{3}{|c}{\\qqqel{}}\\\\\n');
    fprintf(fid_full,'& \\multicolumn{3}{|c|}{\\qqel{}} & \\multicolumn{3}{|c|}{\\qpel{}} & \\multicolumn{3}{|c}{\\qqqel{}}\\\\\n');
     
    fprintf(fid_diag,'$\\alpha$ & $a$ & $b$ & $d$ &');
    fprintf(fid_diag,' $a$ & $b$ & $d$\\\\\n');
    
    fprintf(fid_full,'$\\alpha$ & $a$ & $b$ & $d$ &');
    fprintf(fid_full,'$a$ & $b$ & $d$ & ');
    fprintf(fid_full,' $a$ & $b$ & $d$\\\\\n');
    
    fprintf(fid_diag,'\\hline\n');
    fprintf(fid_full,'\\hline\n');
%     
%     fprintf(fid_diag_bound,'& \\multicolumn{2}{|c|}{\\qqel{}} & \\multicolumn{2}{|c}{\\qqqel{}}\\\\\n');
%     fprintf(fid_full_bound,'& \\multicolumn{2}{|c|}{\\qqel{}} & \\multicolumn{2}{|c|}{\\qpel{}} & \\multicolumn{2}{|c}{\\qqqel{}}\\\\\n');
%     
%     fprintf(fid_diag_bound,'$\\alpha$ & $b$ & Bound& $b$ & Bound\\\\\n');
%     fprintf(fid_full_bound,'$\\alpha$ & $b$ & Bound & $b$ & Bound & $b$ & Bound\\\\\n');
%     
%     fprintf(fid_diag_bound,'\\hline\n');
%     fprintf(fid_full_bound,'\\hline\n');
%     
    %%% Get floating point representations %%%
    [eigmatdiag_exp,eigmatdiag_man] = floating_point(eigmatdiag);
    [eigmatfull_exp,eigmatfull_man] = floating_point(eigmatfull); 
%     [minboundfull_exp,minboundfull_man] = floating_point(minresfaclist(:,:,1)); 
%     [minbounddiag_exp,minbounddiag_man] = floating_point(minresfaclist(:,:,2)); 
    
    for i = 1:numscale
%         fprintf(fid_diag,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & \\fp{%3.2g}{%i}  & ',alphlist(i),eigmatdiag_man(1,i,1),eigmatdiag_exp(1,i,1),eigmatdiag_man(1,i,2),eigmatdiag_exp(1,i,2),eigmatdiag(1,i,4),minbounddiag_man(1,i),minbounddiag_exp(1,i));
%         fprintf(fid_diag,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & \\fp{%3.2g}{%i}  & ',eigmatdiag_man(2,i,1),eigmatdiag_exp(2,i,1),eigmatdiag_man(2,i,2),eigmatdiag_exp(2,i,2),eigmatdiag(2,i,4),minbounddiag_man(2,i),minbounddiag_exp(2,i));
%         fprintf(fid_diag,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & \\fp{%3.2g}{%i}  \\\\\n ',eigmatdiag_man(3,i,1),eigmatdiag_exp(3,i,1),eigmatdiag_man(3,i,2),eigmatdiag_exp(3,i,2),eigmatdiag(3,i,4),minbounddiag_man(3,i),minbounddiag_exp(3,i));
%         fprintf(fid_full,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & \\fp{%3.2g}{%i}  & ',alphlist(i),eigmatfull_man(1,i,1),eigmatfull_exp(1,i,1),eigmatfull_man(1,i,2),eigmatfull_exp(1,i,2),eigmatfull(1,i,4),minboundfull_man(1,i),minboundfull_exp(1,i));
%         fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & \\fp{%3.2g}{%i}  & ',eigmatfull_man(2,i,1),eigmatfull_exp(2,i,1),eigmatfull_man(2,i,2),eigmatfull_exp(2,i,2),eigmatfull(2,i,4),minboundfull_man(2,i),minboundfull_exp(2,i));
%         fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & \\fp{%3.2g}{%i}  \\\\\n ',eigmatfull_man(3,i,1),eigmatfull_exp(3,i,1),eigmatfull_man(3,i,2),eigmatfull_exp(3,i,2),eigmatfull(3,i,4),minboundfull_man(3,i),minboundfull_exp(3,i));
        
        
%         fprintf(fid_diag,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & %3.2g  & ',alphlist(i),eigmatdiag_man(1,i,1),eigmatdiag_exp(1,i,1),eigmatdiag_man(1,i,2),eigmatdiag_exp(1,i,2),eigmatdiag(1,i,4),minresfaclist(1,i,2));
%         fprintf(fid_diag,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & %3.2g  & ',eigmatdiag_man(2,i,1),eigmatdiag_exp(2,i,1),eigmatdiag_man(2,i,2),eigmatdiag_exp(2,i,2),eigmatdiag(2,i,4),minresfaclist(2,i,2));
%         fprintf(fid_diag,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & %3.2g  \\\\\n ',eigmatdiag_man(3,i,1),eigmatdiag_exp(3,i,1),eigmatdiag_man(3,i,2),eigmatdiag_exp(3,i,2),eigmatdiag(3,i,4),minresfaclist(3,i,2));
%         fprintf(fid_full,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & %3.2g  & ',alphlist(i),eigmatfull_man(1,i,1),eigmatfull_exp(1,i,1),eigmatfull_man(1,i,2),eigmatfull_exp(1,i,2),eigmatfull(1,i,4),minresfaclist(1,i,1));
%         fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & %3.2g  & ',eigmatfull_man(2,i,1),eigmatfull_exp(2,i,1),eigmatfull_man(2,i,2),eigmatfull_exp(2,i,2),eigmatfull(2,i,4),minresfaclist(2,i,1));
%         fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g & %3.2g  \\\\\n ',eigmatfull_man(3,i,1),eigmatfull_exp(3,i,1),eigmatfull_man(3,i,2),eigmatfull_exp(3,i,2),eigmatfull(3,i,4),minresfaclist(3,i,1));
%         
        fprintf(fid_diag,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g  & ',alphlist(i),eigmatdiag_man(1,i,1),eigmatdiag_exp(1,i,1),eigmatdiag_man(1,i,2),eigmatdiag_exp(1,i,2),eigmatdiag(1,i,4));
        fprintf(fid_diag,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g  \\\\\n ',eigmatdiag_man(3,i,1),eigmatdiag_exp(3,i,1),eigmatdiag_man(3,i,2),eigmatdiag_exp(3,i,2),eigmatdiag(3,i,4));
        fprintf(fid_full,'%i & \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g  & ',alphlist(i),eigmatfull_man(1,i,1),eigmatfull_exp(1,i,1),eigmatfull_man(1,i,2),eigmatfull_exp(1,i,2),eigmatfull(1,i,4));
        fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g  & ',eigmatfull_man(2,i,1),eigmatfull_exp(2,i,1),eigmatfull_man(2,i,2),eigmatfull_exp(2,i,2),eigmatfull(2,i,4));
        fprintf(fid_full,' \\fp{%3.2g}{%i} & \\fp{%3.2g}{%i} & %3.2g  \\\\\n ',eigmatfull_man(3,i,1),eigmatfull_exp(3,i,1),eigmatfull_man(3,i,2),eigmatfull_exp(3,i,2),eigmatfull(3,i,4));
    end
    fprintf('Done\n');
    
    fprintf('Plotting results...')
    
    cl = get(0,'DefaultAxesColorOrder');
    mo = {'-','--','-.'};
    legendfont = 25;
    labelfont = 30;
    linewidth = 4;
    tickfont = 30;
    
    %%% Full
    figure; hold on
    for pl = 1:3
        plot(alphlist,minresfaclist(pl,:,1)',mo{pl},'color',cl(pl,:),'linewidth',linewidth);
    end
    hold off
    
    set(gca,'xtick',0:20:100);
    h = legend('$Q_1-Q_1$','$Q_1-P_0$','$Q_2-Q_1$','Location','NorthEast');
    set(h,'FontSize',legendfont);
    set(h,'interpreter','latex');
    set(gca,'ticklabelinterpreter','latex')
    set(gca,'fontsize',tickfont);
    ylim([0.4,0.8])

    xlabel('$\alpha$','FontSize',labelfont,'interpreter','latex')
    ylabel('$\eta$','FontSize',labelfont,'interpreter','latex')
    saveas(gcf,['MINRESBoundFull',pb,'.pdf']);
    
    
    %%% Diag
    figure; hold on
    for pl = [1 3]
        plot(alphlist,minresfaclist(pl,:,2)',mo{pl},'color',cl(pl,:),'linewidth',linewidth);
    end
    hold off
    
    set(gca,'xtick',0:20:100);
    h = legend('$Q_1-Q_1$','$Q_2-Q_1$','Location','SouthEast');
    set(h,'FontSize',legendfont);
    set(h,'interpreter','latex');
    set(gca,'ticklabelinterpreter','latex')
    set(gca,'fontsize',tickfont);
    ylim([0.4,0.8])

    xlabel('$\alpha$','FontSize',labelfont,'interpreter','latex')
    ylabel('$\eta$','FontSize',labelfont,'interpreter','latex')
    saveas(gcf,['MINRESBoundDiag',pb,'.pdf']);

%     
    fprintf('Saving results...')
    save([pb,'MINRESBounds.mat'],'alphlist','eigmatdiag','eigmatfull','dellist','minresfaclist','diag_type');
    fprintf('Done\n');
end % for k = 1:numprob

fclose all;
